#ifndef CONTROL_H
#define	CONTROL_H

#include <stdint.h>
#include "esp_err.h"
#include "motors.h"
#include "encoders.h"
#include "hwTimer.h"
#include "math_vector_operations.h"

#define NMOTORS 3
#define MOTOR_PINS {27,26, 33,25, 23,18}
#define MOTOR_ENC  {39,37, 38,36, 34,35}

#define MOTOR_PWM_SUBUNIT_IO {MCPWM0A, MCPWM0B, MCPWM1A, MCPWM1B, MCPWM2A, MCPWM2B}
#define MOTOR_PWM_SUBUNIT_TIMER {MCPWM_TIMER_0, MCPWM_TIMER_1, MCPWM_TIMER_2}
#define MOTOR_PWM_UNIT MCPWM_UNIT_0
#define OMNI_PID_NRO_TIMER 0
#define OMNI_ODOM_NRO_TIMER 2

#define OMNI_PID_CTRL_TIMER 0.05 // in seconds
#define OMNI_ODOM_CTRL_TIMER 0.5 // in seconds

#define MAX_OUTPUT 100.0
#define WHEEL_DIAMETER 0.038 // in meters
#define WHEEL_RADIUS  WHEEL_DIAMETER/2// in meters
#define ROBOT_RADIUS 0.0675 // in meters

#define ENC_CPR 12 //counts per revolution
#define MOTOR_REDUCTION 50
#define TICS_PER_REVOLUTION ENC_CPR*MOTOR_REDUCTION
#define RAD_PER_TICK 2*PI/TICS_PER_REVOLUTION
#define M_PER_SEC_TO_TICS_PER_SEC 1/(RAD_PER_TICK*WHEEL_RADIUS)

#define MAX_SPEED_POWER 90 // power % at which MAX_SPEED_TICS is obtained
#define MAX_SPEED_TICS 1080 //tics/s at MAX_SPEED_POWER

#define MAX_SPEED_RAD MAX_SPEED_TICS*RAD_PER_TICK  // rad/s
#define MAX_SPEED_LIN MAX_SPEED_RAD*WHEEL_DIAMETER / 2 // m/s

#define TICS_TO_RAD_S RAD_PER_TICK/OMNI_ODOM_CTRL_TIMER

#define KP 0.01
#define KI 0.05
#define KD 0.0
#define KF MAX_SPEED_POWER/MAX_SPEED_TICS

typedef struct {

	motor_h_t *driver;
    encoder_h_t *encoder;

    int32_t vel_counter;
    int32_t odom_counter;

    float target_v;
    float accum_error;
    float prev_error;
    float output;

} servo_t;

typedef struct {
    float x;
    float x_dot;
    float y;
    float y_dot;
    float phi;
    float phi_dot;

} odom_t;



static servo_t motors[NMOTORS];
static odom_t odometry;

esp_err_t omni_init();

esp_err_t omni_set_enable(bool start);

esp_err_t omni_drive(float x_dot ,float y_dot ,float w_dot ,float phi);

#endif /* CONTROL_H */